var Install = function()
{
	var install = {};
	
	var checks;
	var debug = 0;
	
	install.initPage = function()
	{
		Events.addEventHandler('typeFilter', 'change', install.filterOnChange);
		Events.addEventHandler('nameFilter', 'change', install.filterOnChange);
		Events.addEventHandler('messageFilter', 'change', install.filterOnChange);
		
		Buttons.addClickHandler('btnFixItems', install.fixInstallation);
		Buttons.addClickHandler('btnCreateMissingItems', install.fixInstallation);
		
		install.checkInstallation('Validate', 'Checking installation...');
	}
	
	install.fixInstallation = function()
	{
		install.checkInstallation('Install', 'Fixing installation...');
	}

	
	install.checkInstallation = function(cmd, workText, callback, progress)
	{
		Doc.hide('item-list');
		Doc.hide('itemTable');
		Doc.hide('createMissingItems');
		Doc.hide('fixIncompleteItems');
		
		var url = '../Install.servlet?ID=' + App.getSessionId();
		url += '&cmd='+cmd;
		Doc.addClass('workInProgress', 'working');
		Wizard.showLoadingAnimation(workText, progress);
		Wizard.asyncJsonRequest(url, callback || install.onChecked);
	}
	
	install.onChecked = function(response)
	{
		Doc.removeClass('workInProgress', 'working');
		
		checks = response.checks;
		
		var numMissing = 0;
		var numWarnings = 0;
		var numErrors = 0;
		var numIncomplete = 0;
		
		var index = 0;
		var topLines = []; // Top lines are for items with error/warning/special message
		var bottomLines = []; // Bottom lines are for all 'OK' items
		var allOk = true;
		var clickableItems = [];
		var typeFilter = Doc.element('typeFilter');
		typeFilter.length = 0;
		typeFilter[0] = new Option();
		var types = [];
		
		for (var checkNo = 0; checkNo < checks.length; checkNo++)
		{
			var check = checks[checkNo];
			
			var icon = 'ok.png';
			check.ok = true;
			if (check.status == 'missing') 
			{
				numMissing++;
				icon = 'warning.png';
				check.ok = false;
			}
			if (check.status == 'incomplete')
			{
				numIncomplete++;
				icon = 'warning.png';
				check.ok = false;
			}
			if (check.status == 'error') 
			{
				numErrors++;
				icon = 'error.png';
				check.ok = false;
			}
			if (check.status == 'warning') 
			{
				numWarnings++;
				icon = 'warning.png';
				check.ok = false;
			}
			allOk &= check.ok;
			
			if (!types[check.itemType])
			{
				typeFilter[typeFilter.length] = new Option(check.itemType);
				types[check.itemType] = 1;
			}
			
			var line = '<tr id="check'+checkNo+'" class="highlight '+check.itemType+' ' + (check.ok ? 'check-ok' : 'check-not-ok') + '">';
			line += '<td class="indexCol"><span id="index.'+checkNo+'">'+(checkNo+1)+'</span></td>';
			line += '<td class="dottedleft">'+Strings.encodeTags(check.itemType)+'</td>';
			var name = Strings.encodeTags(check.name);
			if (check.mainType) name += ' <span class="itemsubtype">[' + Strings.encodeTags(check.mainType) + ']</span>'; 
			if (check.id)
			{
				clickableItems[clickableItems.length] = check.itemType+check.id;
				line += '<td><div class="link" id="'+check.itemType+check.id+'" data-item-type="'+check.itemType+'" data-item-id="'+check.id+'"';
				line += ' title="View this item (use CTRL, ALT or SHIFT to edit)">'+name+'</div></td>';
			}
			else
			{
				line += '<td><i>' + name + '</i></td>';
			}
			line += '<td class="iconCol"><img src="../images/'+icon+'"></td>';
			line += '<td>';
			if (check.messages.length > 1)
			{
				for (var m = 0; m < check.messages.length; m++)
				{
					line += '• '+check.messages[m] + '<br>';
				}
			}
			else
			{
				line += check.messages;
			}
			line += '</td></tr>';
			
			if (!check.ok || check.messages != 'Ok')
			{
				topLines[topLines.length] = line;
			}
			else
			{
				bottomLines[bottomLines.length] = line;
			}
			index++;
		}
		
		if (topLines.length + bottomLines.length > 0)
		{
			Doc.element('item-list').innerHTML = topLines.join('') + bottomLines.join('');
			Doc.show('item-list');
		}
		Doc.show('itemTable', 'table');

		// Add click handler to items
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', Items.itemOnClick);
		}
		
		if (numErrors > 0)
		{
			Wizard.setWizardStatus('messagecontainer error', numErrors+' errors was detected. You need to fix those manually.');	
		}
		else if (numWarnings > 0)
		{
			Wizard.setWizardStatus('messagecontainer note', numWarnings+' warnings was detected. Relax may still work. If not, you need to fix it manually.');
		}
		if (numMissing > 0)
		{
			Doc.show('createMissingItems');
		}
		else if (numIncomplete > 0)
		{
			Doc.show('fixIncompleteItems');
		}
		if (allOk) 
		{
			Wizard.setWizardStatus('all-checks-passed', 'All checks passed');
		}
	}
	
	install.showMessages = function(response)
	{
		Doc.removeClass('workInProgress', 'working');
		Wizard.showFinalMessage(response.messages);
	}
	
	install.filterOnChange = function()
	{
		
		var filter = new Filter();
		filter.addTextCriteria(ItemProperty.ItemType, Doc.element('typeFilter').value);
		filter.addContainsCriteria(ItemProperty.Name, Doc.element('nameFilter').value);
		filter.addContainsCriteria(ItemProperty.Messages, Doc.element('messageFilter').value);

		var numMatch = 0;
		for (var checkNo = 0; checkNo < checks.length; checkNo++)
		{
			var check = checks[checkNo];
			
			check.include = filter.check(check);
			if (check.include)
			{
				numMatch++;
				Doc.show('check'+checkNo);
			}
			else
			{
				Doc.hide('check'+checkNo);
			}
		}
		
		Doc.showHide('item-list', numMatch > 0);
		Doc.showHide('no-matching-items', numMatch==0);
	}
	
	return install;
}();

Doc.onLoad(Install.initPage);

var ItemProperty = function()
{
	var p = {};
	
	p.Name = function(item)
	{
		return item.name;
	}
	
	p.ItemType = function(item)
	{
		return item.itemType;
	}

	p.Messages = function(item)
	{
		return item.messages.join('');
	}
	
	return p;
}();

